from kaleo.models import JoinInvitation


def stats():
    return {
        "join_invitations_sent": JoinInvitation.objects.count(),
        "join_invitations_accepted": JoinInvitation.objects.filter(status=JoinInvitation.STATUS_ACCEPTED).count()
    }



import sys

from django.core.management.base import BaseCommand

from django.contrib.auth.models import User

from kaleo.models import InvitationStat


class Command(BaseCommand):
    help = "Makes sure all users have a certain number of invites."
    
    def handle(self, *args, **kwargs):
        if len(args) == 0:
            sys.exit("You must supply the number of invites as an argument.")
        
        try:
            num_of_invites = int(args[0])
        except ValueError:
            sys.exit("The argument for number of invites must be an integer.")
        
        for user in User.objects.all():
            stat, _ = InvitationStat.objects.get_or_create(user=user)
            remaining = stat.invites_remaining()
            if remaining < num_of_invites:
                stat.invites_allocated += (num_of_invites - remaining)
                stat.save()



from django import template

from kaleo.forms import InviteForm
from kaleo.models import InvitationStat


register = template.Library()


class RemainingInvitesNode(template.Node):
    
    @classmethod
    def handle_token(cls, parser, token):
        bits = token.split_contents()
        if len(bits) != 2:
            raise template.TemplateSyntaxError
        
        return cls(
            user = parser.compile_filter(bits[1])
        )
    
    def __init__(self, user):
        self.user = user
    
    def render(self, context):
        user = self.user.resolve(context)
        try:
            return user.invitationstat.invites_remaining()
        except InvitationStat.DoesNotExist:
            return "0"


@register.tag
def remaining_invites(parser, token):
    """
    Usage::
        {% remaining_invites user %}
    
    Returns an integer that is the # of remaining invites the user has.
    """
    return RemainingInvitesNode.handle_token(parser, token)


@register.inclusion_tag("kaleo/_invite_form.html")
def invite_form(user):
    return {"form": InviteForm(), "user": user}


@register.inclusion_tag("kaleo/_invited.html")
def invites_sent(user):
    return {"invited_list": user.invites_sent.all()}


@register.filter
def status_class(invite):
    if invite.status == invite.STATUS_SENT:
        return "sent"
    elif invite.status == invite.STATUS_ACCEPTED:
        return "accepted"
    elif invite.status == invite.STATUS_JOINED_INDEPENDENTLY:
        return "joined"
    return ""



from django.db import models
from django.db.models.signals import post_save
from django.conf import settings
from django.dispatch import receiver
from django.utils import timezone

from django.contrib.auth.models import User

from account.models import SignupCode, SignupCodeResult, EmailConfirmation
from account.signals import signup_code_used, email_confirmed
from kaleo.signals import invite_sent, invite_accepted


DEFAULT_INVITE_EXPIRATION = getattr(settings, "KALEO_DEFAULT_EXPIRATION", 168) # 168 Hours = 7 Days
DEFAULT_INVITE_ALLOCATION = getattr(settings, "KALEO_DEFAULT_INVITE_ALLOCATION", 0)


class NotEnoughInvitationsError(Exception):
    pass


class JoinInvitation(models.Model):
    
    STATUS_SENT = 1
    STATUS_ACCEPTED = 2
    STATUS_JOINED_INDEPENDENTLY = 3
    
    INVITE_STATUS_CHOICES = [
        (STATUS_SENT, "Sent"),
        (STATUS_ACCEPTED, "Accepted"),
        (STATUS_JOINED_INDEPENDENTLY, "Joined Independently")
    ]
    
    from_user = models.ForeignKey(User, related_name="invites_sent")
    to_user = models.ForeignKey(User, null=True, related_name="invites_received")
    message = models.TextField(null=True)
    sent = models.DateTimeField(default=timezone.now)
    status = models.IntegerField(choices=INVITE_STATUS_CHOICES)
    signup_code = models.OneToOneField(SignupCode)
    
    def to_user_email(self):
        return self.signup_code.email
    
    @classmethod
    def invite(cls, from_user, to_email, message=None):
        if not from_user.invitationstat.can_send():
            raise NotEnoughInvitationsError()
        
        signup_code = SignupCode.create(
            email=to_email,
            inviter=from_user,
            expiry=DEFAULT_INVITE_EXPIRATION,
            check_exists=False # before we are called caller must check for existence
        )
        signup_code.save()
        join = cls.objects.create(
            from_user=from_user,
            message=message,
            status=JoinInvitation.STATUS_SENT,
            signup_code=signup_code
        )
        signup_code.send()
        stat = from_user.invitationstat
        stat.invites_sent += 1
        stat.save()
        invite_sent.send(sender=cls, invitation=join)
        return join


class InvitationStat(models.Model):
    
    user = models.OneToOneField(User)
    invites_sent = models.IntegerField(default=0)
    invites_allocated = models.IntegerField(default=DEFAULT_INVITE_ALLOCATION)
    invites_accepted = models.IntegerField(default=0)
    
    def invites_remaining(self):
        if self.invites_allocated == -1:
            return -1
        return self.invites_allocated - self.invites_sent
    
    def can_send(self):
        if self.invites_allocated == -1:
            return True
        return self.invites_allocated > self.invites_sent
    can_send.boolean = True


@receiver(signup_code_used, sender=SignupCodeResult)
def handle_signup_code_used(sender, **kwargs):
    result = kwargs.get("signup_code_result")
    try:
        invite = result.signup_code.joininvitation
        invite.to_user = result.user
        invite.status = JoinInvitation.STATUS_ACCEPTED
        invite.save()
        stat = invite.from_user.invitationstat
        stat.invites_accepted += 1
        stat.save()
        invite_accepted.send(sender=JoinInvitation, invitation=invite)
    except JoinInvitation.DoesNotExist:
        pass


@receiver(email_confirmed, sender=EmailConfirmation)
def handle_email_confirmed(sender, **kwargs):
    email_address = kwargs.get("email_address")
    invites = JoinInvitation.objects.filter(
        to_user__isnull=True,
        signup_code__email=email_address.email
    )
    for invite in invites:
        invite.to_user = email_address.user
        invite.status = JoinInvitation.STATUS_JOINED_INDEPENDENTLY
        invite.save()


@receiver(post_save, sender=User)
def create_stat(sender, instance=None, **kwargs):
    if instance is None:
        return
    InvitationStat.objects.get_or_create(user=instance)



from django.contrib import admin

from kaleo.models import JoinInvitation, InvitationStat


class InvitationStatAdmin(admin.ModelAdmin):
    raw_id_fields = ["user"]
    readonly_fields = ["invites_sent", "invites_accepted"]
    list_display = ["user", "invites_sent", "invites_accepted", "invites_allocated", "invites_remaining", "can_send"]
    list_filter = ["invites_sent", "invites_accepted"]


admin.site.register(JoinInvitation,
    list_display = ["from_user", "to_user", "sent", "status", "to_user_email"],
    list_filter = ["sent", "status"],
    search_fields = ["from_user__username"]
)
admin.site.register(InvitationStat, InvitationStatAdmin)



from django import http
from django.utils import simplejson as json
from django.views.decorators.http import require_http_methods

from django.contrib.auth.decorators import login_required

from account.models import EmailAddress
from kaleo.forms import InviteForm
from kaleo.models import JoinInvitation


@login_required
@require_http_methods(["POST"])
def invite(request):
    form = InviteForm(request.POST)
    if form.is_valid():
        email = form.cleaned_data["email_address"]
        if EmailAddress.objects.filter(email=email, verified=True).exists():
            data = {"status": "ERROR", "errors": '<ul class="errorlist"><li>%s is already on this site</li></ul>' % email}
        elif JoinInvitation.objects.filter(from_user=request.user, signup_code__email=email).exists():
            data = {"status": "ERROR", "errors": '<ul class="errorlist"><li>You have already invited %s</li></ul>' % email}
        else:
            JoinInvitation.invite(request.user, email)
            data = {
                "status": "OK",
                "email": email,
                "invitations_remaining": request.user.invitationstat.invites_remaining()
            }
    else:
        data = {"status": "ERROR", "errors": str(form.errors["email_address"])}
    return http.HttpResponse(json.dumps(data), content_type="application/json")
